@@ -32,13 +32,15 @@
 
 #include "helpers.h"
 #include "catch.hpp"
+#include "serialization_wrappers.h"
 #include "tiledb/common/logger.h"
+#include "tiledb/sm/c_api/tiledb_struct_def.h"
 #include "tiledb/sm/cpp_api/tiledb"
 #include "tiledb/sm/enums/encryption_type.h"
+#include "tiledb/sm/filesystem/uri.h"
 #include "tiledb/sm/global_state/unit_test_config.h"
 #include "tiledb/sm/misc/constants.h"
-#include "tiledb/sm/misc/uri.h"
-#include "tiledb/sm/subarray/subarray_partitioner.h"
+#include "tiledb/sm/misc/tile_overlap.h"
 
 std::mutex catch2_macro_mutex;
 
@@ -46,7 +48,20 @@ namespace tiledb {
 namespace test {
 
 // Command line arguments.
-extern std::string g_vfs;
+std::string g_vfs;
+
+int store_g_vfs(std::string&& vfs, std::vector<std::string> vfs_fs) {
+  if (!vfs.empty()) {
+    if (std::find(vfs_fs.begin(), vfs_fs.end(), vfs) == vfs_fs.end()) {
+      std::cerr << "Unknown --vfs argument: \"" << vfs << "\"";
+      return 1;
+    }
+
+    tiledb::test::g_vfs = std::move(vfs);
+  }
+
+  return 0;
+}
 
 bool use_refactored_dense_reader() {
   const char* value = nullptr;
@@ -107,8 +122,8 @@ bool use_refactored_sparse_unordered_with_dups_reader() {
   return use_refactored_readers;
 }
 
-tdb_shared_ptr<Logger> g_helper_logger(void) {
-  static tdb_shared_ptr<Logger> g_helper_logger = tdb_make_shared(Logger, "");
+shared_ptr<Logger> g_helper_logger(void) {
+  static shared_ptr<Logger> g_helper_logger = make_shared<Logger>(HERE(), "");
   return g_helper_logger;
 }
 
@@ -171,7 +186,7 @@ void check_subarray(
 
   // Check ranges
   uint64_t dim_range_num = 0;
-  const sm::Range* range;
+  const type::Range* range;
   for (unsigned i = 0; i < dim_num; ++i) {
     CHECK(subarray.get_range_num(i, &dim_range_num).ok());
     CHECK(dim_range_num == ranges[i].size() / 2);
@@ -185,74 +200,148 @@ void check_subarray(
   }
 }
 
+template <class T>
+void check_subarray(
+    tiledb::Subarray& subarray, const SubarrayRanges<T>& ranges) {
+  auto as = subarray.array().schema();
+  auto ndims = as.domain().ndim();
+  uint64_t nranges = 1;
+  for (auto ui = 0u; ui < ndims; ++ui) {
+    auto range_num_dim = subarray.range_num(ui);
+    nranges *= range_num_dim;
+  }
+  // Check empty subarray
+  auto subarray_range_num = nranges;
+  if (ranges.empty()) {
+    CHECK(subarray_range_num == 0);
+    return;
+  }
+  uint64_t range_num = 1;
+  for (const auto& dim_ranges : ranges)
+    range_num *= dim_ranges.size() / 2;
+  CHECK(subarray_range_num == range_num);
+
+  // Check dim num
+  auto dim_num = ndims;
+  CHECK(dim_num == ranges.size());
+
+  // Check ranges
+  uint64_t dim_range_num = 0;
+  for (unsigned di = 0; di < dim_num; ++di) {
+    dim_range_num = subarray.range_num(di);
+    CHECK(dim_range_num == ranges[di].size() / 2);
+    for (uint64_t ri = 0; ri < dim_range_num; ++ri) {
+      auto r = subarray.range<T>(di, ri);
+
+      CHECK(r[0] == ranges[di][2 * ri]);
+      CHECK(r[1] == ranges[di][2 * ri + 1]);
+    }
+  }
+}
+
+template <class T>
+void check_subarray_equiv(
+    tiledb::sm::Subarray& subarray1, tiledb::sm::Subarray& subarray2) {
+  CHECK(subarray1.range_num() == subarray2.range_num());
+  // Check dim num
+  auto dim_num1 = subarray1.dim_num();
+  auto dim_num2 = subarray2.dim_num();
+  CHECK(dim_num1 == dim_num2);
+
+  tiledb::sm::ByteVec sa1bytes, sa2bytes;
+  //.to_byte_vect() only valid when range_num() == 1, but should be same for
+  // both and resulting bytes, empty or otherwise, should be the same as well.
+  CHECK(
+      subarray1.to_byte_vec(&sa1bytes).ok() ==
+      subarray2.to_byte_vec(&sa2bytes).ok());
+  CHECK(sa1bytes == sa2bytes);
+
+  const std::vector<std::vector<uint8_t>> sa1tilecoords =
+      subarray1.tile_coords();
+  const std::vector<std::vector<uint8_t>> sa2tilecoords =
+      subarray2.tile_coords();
+  CHECK(sa1tilecoords == sa2tilecoords);
+
+  // Check ranges
+  uint64_t dim_range_num1 = 0;
+  const type::Range* range1;
+  uint64_t dim_range_num2 = 0;
+  const type::Range* range2;
+  if (dim_num1 == dim_num2) {
+    for (unsigned i = 0; i < dim_num1; ++i) {
+      CHECK(subarray1.get_range_num(i, &dim_range_num1).ok());
+      CHECK(subarray2.get_range_num(i, &dim_range_num2).ok());
+      CHECK(dim_range_num1 == dim_range_num2);
+      if (dim_range_num1 == dim_range_num2) {
+        for (uint64_t j = 0; j < dim_range_num1; ++j) {
+          subarray1.get_range(i, j, &range1);
+          subarray2.get_range(i, j, &range2);
+          auto r1 = (const T*)range1->data();
+          auto r2 = (const T*)range2->data();
+          CHECK(r1[0] == r2[0]);
+          CHECK(r1[1] == r2[1]);
+        }
+      }
+    }
+  }
+}
+
+template <class T>
+bool subarray_equiv(
+    tiledb::sm::Subarray& subarray1, tiledb::sm::Subarray& subarray2) {
+  bool equiv_state = 1;  // assume true
+
+  equiv_state &= (subarray1.range_num() == subarray2.range_num());
+  // Check dim num
+  auto dim_num1 = subarray1.dim_num();
+  auto dim_num2 = subarray2.dim_num();
+  equiv_state &= (dim_num1 == dim_num2);
+
+  tiledb::sm::ByteVec sa1bytes, sa2bytes;
+  //.to_byte_vect() only valid when range_num() == 1, but should be same for
+  // both and resulting bytes, empty or otherwise, should be the same as well.
+  equiv_state &=
+      (subarray1.to_byte_vec(&sa1bytes).ok() ==
+       subarray2.to_byte_vec(&sa2bytes).ok());
+  equiv_state &= (sa1bytes == sa2bytes);
+
+  const std::vector<std::vector<uint8_t>> sa1tilecoords =
+      subarray1.tile_coords();
+  const std::vector<std::vector<uint8_t>> sa2tilecoords =
+      subarray2.tile_coords();
+  CHECK(sa1tilecoords == sa2tilecoords);
+
+  // Check ranges
+  uint64_t dim_range_num1 = 0;
+  const type::Range* range1;
+  uint64_t dim_range_num2 = 0;
+  const type::Range* range2;
+  if (dim_num1 == dim_num2) {
+    for (unsigned i = 0; i < dim_num1; ++i) {
+      equiv_state &= (subarray1.get_range_num(i, &dim_range_num1).ok());
+      equiv_state &= (subarray2.get_range_num(i, &dim_range_num2).ok());
+      equiv_state &= (dim_range_num1 == dim_range_num2);
+      if (dim_range_num1 == dim_range_num2) {
+        for (uint64_t j = 0; j < dim_range_num1; ++j) {
+          subarray1.get_range(i, j, &range1);
+          subarray2.get_range(i, j, &range2);
+          auto r1 = (const T*)range1->data();
+          auto r2 = (const T*)range2->data();
+          equiv_state &= (r1[0] == r2[0]);
+          equiv_state &= (r1[1] == r2[1]);
+        }
+      }
+    }
+  }
+
+  return equiv_state;
+}
+
 void close_array(tiledb_ctx_t* ctx, tiledb_array_t* array) {
   int rc = tiledb_array_close(ctx, array);
   CHECK(rc == TILEDB_OK);
 }
 
-int array_create_wrapper(
-    tiledb_ctx_t* ctx,
-    const std::string& path,
-    tiledb_array_schema_t* array_schema,
-    bool serialize_array_schema) {
-#ifndef TILEDB_SERIALIZATION
-  return tiledb_array_create(ctx, path.c_str(), array_schema);
-#endif
-
-  if (!serialize_array_schema) {
-    return tiledb_array_create(ctx, path.c_str(), array_schema);
-  }
-
-  // Serialize the array
-  tiledb_buffer_t* buff;
-  REQUIRE(
-      tiledb_serialize_array_schema(
-          ctx,
-          array_schema,
-          (tiledb_serialization_type_t)tiledb::sm::SerializationType::CAPNP,
-          1,
-          &buff) == TILEDB_OK);
-
-  // Load array schema from the rest server
-  tiledb_array_schema_t* new_array_schema = nullptr;
-  REQUIRE(
-      tiledb_deserialize_array_schema(
-          ctx,
-          buff,
-          (tiledb_serialization_type_t)tiledb::sm::SerializationType::CAPNP,
-          0,
-          &new_array_schema) == TILEDB_OK);
-
-  // Create array from new schema
-  int rc = tiledb_array_create(ctx, path.c_str(), new_array_schema);
-
-  // Serialize the new array schema and deserialize into the original array
-  // schema.
-  tiledb_buffer_t* buff2;
-  REQUIRE(
-      tiledb_serialize_array_schema(
-          ctx,
-          new_array_schema,
-          (tiledb_serialization_type_t)tiledb::sm::SerializationType::CAPNP,
-          0,
-          &buff2) == TILEDB_OK);
-  REQUIRE(
-      tiledb_deserialize_array_schema(
-          ctx,
-          buff2,
-          (tiledb_serialization_type_t)tiledb::sm::SerializationType::CAPNP,
-          1,
-          &array_schema) == TILEDB_OK);
-
-  // Clean up.
-  tiledb_array_schema_free(&array_schema);
-  tiledb_array_schema_free(&new_array_schema);
-  tiledb_buffer_free(&buff);
-  tiledb_buffer_free(&buff2);
-
-  return rc;
-}
-
 void create_array(
     tiledb_ctx_t* ctx,
     const std::string& array_name,
@@ -339,7 +428,7 @@ void create_array(
   REQUIRE(rc == TILEDB_OK);
 
   // Create array
-  rc = array_create_wrapper(
+  rc = tiledb_array_create_serialization_wrapper(
       ctx, array_name, array_schema, serialize_array_schema);
   REQUIRE(rc == TILEDB_OK);
 
@@ -573,7 +662,7 @@ void create_subarray(
   for (unsigned d = 0; d < dim_num; ++d) {
     auto dim_range_num = ranges[d].size() / 2;
     for (size_t j = 0; j < dim_range_num; ++j) {
-      sm::Range range(&ranges[d][2 * j], 2 * sizeof(T));
+      type::Range range(&ranges[d][2 * j], 2 * sizeof(T));
       ret.add_range(d, std::move(range), true);
     }
   }
@@ -581,6 +670,61 @@ void create_subarray(
   *subarray = ret;
 }
 
+template <class T>
+void create_subarray(
+    tiledb_ctx_t* ctx,
+    tiledb::sm::Array* array,
+    const SubarrayRanges<T>& ranges,
+    tiledb::sm::Layout layout,
+    tiledb_subarray_t** subarray,
+    bool coalesce_ranges) {
+  (void)layout;
+  int32_t rc;
+  tiledb_array_t tdb_array;
+  tdb_array.array_ = array;
+  rc = tiledb_subarray_alloc(ctx, &tdb_array, subarray);
+  REQUIRE(rc == TILEDB_OK);
+  if (rc == TILEDB_OK) {
+    rc = tiledb_subarray_set_coalesce_ranges(ctx, *subarray, coalesce_ranges);
+    REQUIRE(rc == TILEDB_OK);
+
+    auto dim_num = (unsigned)ranges.size();
+    for (unsigned d = 0; d < dim_num; ++d) {
+      auto dim_range_num = ranges[d].size() / 2;
+      for (size_t j = 0; j < dim_range_num; ++j) {
+        rc = tiledb_subarray_add_range(
+            ctx, *subarray, d, &ranges[d][2 * j], &ranges[d][2 * j + 1], 0);
+        REQUIRE(rc == TILEDB_OK);
+      }
+    }
+  }
+}
+
+template <class T>
+void create_subarray(
+    const tiledb::Context* ctx,
+    const tiledb::Array* array,
+    const SubarrayRanges<T>& ranges,
+    tiledb::sm::Layout layout,
+    tiledb::Subarray** returned_subarray,
+    bool coalesce_ranges) {
+  tiledb::Subarray* psubarray =
+      new tiledb::Subarray(*ctx, *array, coalesce_ranges);
+  tiledb::Subarray& subarray = *psubarray;
+
+  (void)layout;
+  subarray.set_coalesce_ranges(coalesce_ranges);
+
+  auto dim_num = (unsigned)ranges.size();
+  for (unsigned d = 0; d < dim_num; ++d) {
+    auto dim_range_num = ranges[d].size() / 2;
+    for (size_t j = 0; j < dim_range_num; ++j) {
+      subarray.add_range<T>(d, ranges[d][2 * j], ranges[d][2 * j + 1], 0);
+    }
+  }
+  *returned_subarray = psubarray;
+}
+
 void get_supported_fs(
     bool* s3_supported,
     bool* hdfs_supported,
@@ -1027,19 +1171,32 @@ int32_t num_fragments(const std::string& array_name) {
   VFS vfs(ctx);
 
   // Get all URIs in the array directory
-  auto uris = vfs.ls(array_name);
+  auto uris = vfs.ls(
+      array_name + "/" + tiledb::sm::constants::array_fragments_dir_name);
+  return static_cast<uint32_t>(uris.size());
+}
 
-  // Exclude '__meta' folder and any file with a suffix
-  int ret = 0;
-  for (const auto& uri : uris) {
-    auto name = tiledb::sm::URI(uri).remove_trailing_slash().last_path_part();
-    if (name != tiledb::sm::constants::array_metadata_folder_name &&
-        name != tiledb::sm::constants::array_schema_folder_name &&
-        name.find_first_of('.') == std::string::npos)
-      ++ret;
+std::string random_string(const uint64_t l) {
+  static const char char_set[] =
+      "0123456789"
+      "ABCDEFGHIJKLMNOPQRSTUVWXYZ"
+      "abcdefghijklmnopqrstuvwxyz";
+  std::string s;
+  s.reserve(l);
+
+  for (uint64_t i = 0; i < l; ++i) {
+    s += char_set[rand() % (sizeof(char_set) - 1)];
   }
 
-  return ret;
+  return s;
+}
+
+std::string get_fragment_dir(std::string array_dir) {
+  return array_dir + "/" + tiledb::sm::constants::array_fragments_dir_name;
+}
+
+std::string get_commit_dir(std::string array_dir) {
+  return array_dir + "/" + tiledb::sm::constants::array_commits_dir_name;
 }
 
 template void check_subarray<int8_t>(
@@ -1063,6 +1220,48 @@ template void check_subarray<float>(
 template void check_subarray<double>(
     tiledb::sm::Subarray& subarray, const SubarrayRanges<double>& ranges);
 
+template void check_subarray_equiv<int8_t>(
+    tiledb::sm::Subarray& subarray1, tiledb::sm::Subarray& subarray2);
+template void check_subarray_equiv<uint8_t>(
+    tiledb::sm::Subarray& subarray1, tiledb::sm::Subarray& subarray2);
+template void check_subarray_equiv<int16_t>(
+    tiledb::sm::Subarray& subarray1, tiledb::sm::Subarray& subarray2);
+template void check_subarray_equiv<uint16_t>(
+    tiledb::sm::Subarray& subarray1, tiledb::sm::Subarray& subarray2);
+template void check_subarray_equiv<int32_t>(
+    tiledb::sm::Subarray& subarray1, tiledb::sm::Subarray& subarray2);
+template void check_subarray_equiv<uint32_t>(
+    tiledb::sm::Subarray& subarray1, tiledb::sm::Subarray& subarray2);
+template void check_subarray_equiv<int64_t>(
+    tiledb::sm::Subarray& subarray1, tiledb::sm::Subarray& subarray2);
+template void check_subarray_equiv<uint64_t>(
+    tiledb::sm::Subarray& subarray1, tiledb::sm::Subarray& subarray2);
+template void check_subarray_equiv<float>(
+    tiledb::sm::Subarray& subarray1, tiledb::sm::Subarray& subarray2);
+template void check_subarray_equiv<double>(
+    tiledb::sm::Subarray& subarray1, tiledb::sm::Subarray& subarray2);
+
+template bool subarray_equiv<int8_t>(
+    tiledb::sm::Subarray& subarray1, tiledb::sm::Subarray& subarray2);
+template bool subarray_equiv<uint8_t>(
+    tiledb::sm::Subarray& subarray1, tiledb::sm::Subarray& subarray2);
+template bool subarray_equiv<int16_t>(
+    tiledb::sm::Subarray& subarray1, tiledb::sm::Subarray& subarray2);
+template bool subarray_equiv<uint16_t>(
+    tiledb::sm::Subarray& subarray1, tiledb::sm::Subarray& subarray2);
+template bool subarray_equiv<int32_t>(
+    tiledb::sm::Subarray& subarray1, tiledb::sm::Subarray& subarray2);
+template bool subarray_equiv<uint32_t>(
+    tiledb::sm::Subarray& subarray1, tiledb::sm::Subarray& subarray2);
+template bool subarray_equiv<int64_t>(
+    tiledb::sm::Subarray& subarray1, tiledb::sm::Subarray& subarray2);
+template bool subarray_equiv<uint64_t>(
+    tiledb::sm::Subarray& subarray1, tiledb::sm::Subarray& subarray2);
+template bool subarray_equiv<float>(
+    tiledb::sm::Subarray& subarray1, tiledb::sm::Subarray& subarray2);
+template bool subarray_equiv<double>(
+    tiledb::sm::Subarray& subarray1, tiledb::sm::Subarray& subarray2);
+
 template void create_subarray<int8_t>(
     tiledb::sm::Array* array,
     const SubarrayRanges<int8_t>& ranges,
@@ -1124,6 +1323,148 @@ template void create_subarray<double>(
     tiledb::sm::Subarray* subarray,
     bool coalesce_ranges);
 
+template void create_subarray<int8_t>(
+    tiledb_ctx_t* ctx,
+    tiledb::sm::Array* array,
+    const SubarrayRanges<int8_t>& ranges,
+    tiledb::sm::Layout layout,
+    tiledb_subarray_t** subarray,
+    bool coalesce_ranges);
+template void create_subarray<uint8_t>(
+    tiledb_ctx_t* ctx,
+    tiledb::sm::Array* array,
+    const SubarrayRanges<uint8_t>& ranges,
+    tiledb::sm::Layout layout,
+    tiledb_subarray_t** subarray,
+    bool coalesce_ranges);
+template void create_subarray<int16_t>(
+    tiledb_ctx_t* ctx,
+    tiledb::sm::Array* array,
+    const SubarrayRanges<int16_t>& ranges,
+    tiledb::sm::Layout layout,
+    tiledb_subarray_t** subarray,
+    bool coalesce_ranges);
+template void create_subarray<uint16_t>(
+    tiledb_ctx_t* ctx,
+    tiledb::sm::Array* array,
+    const SubarrayRanges<uint16_t>& ranges,
+    tiledb::sm::Layout layout,
+    tiledb_subarray_t** subarray,
+    bool coalesce_ranges);
+template void create_subarray<int32_t>(
+    tiledb_ctx_t* ctx,
+    tiledb::sm::Array* array,
+    const SubarrayRanges<int32_t>& ranges,
+    tiledb::sm::Layout layout,
+    tiledb_subarray_t** subarray,
+    bool coalesce_ranges);
+template void create_subarray<uint32_t>(
+    tiledb_ctx_t* ctx,
+    tiledb::sm::Array* array,
+    const SubarrayRanges<uint32_t>& ranges,
+    tiledb::sm::Layout layout,
+    tiledb_subarray_t** subarray,
+    bool coalesce_ranges);
+template void create_subarray<int64_t>(
+    tiledb_ctx_t* ctx,
+    tiledb::sm::Array* array,
+    const SubarrayRanges<int64_t>& ranges,
+    tiledb::sm::Layout layout,
+    tiledb_subarray_t** subarray,
+    bool coalesce_ranges);
+template void create_subarray<uint64_t>(
+    tiledb_ctx_t* ctx,
+    tiledb::sm::Array* array,
+    const SubarrayRanges<uint64_t>& ranges,
+    tiledb::sm::Layout layout,
+    tiledb_subarray_t** subarray,
+    bool coalesce_ranges);
+template void create_subarray<float>(
+    tiledb_ctx_t* ctx,
+    tiledb::sm::Array* array,
+    const SubarrayRanges<float>& ranges,
+    tiledb::sm::Layout layout,
+    tiledb_subarray_t** subarray,
+    bool coalesce_ranges);
+template void create_subarray<double>(
+    tiledb_ctx_t* ctx,
+    tiledb::sm::Array* array,
+    const SubarrayRanges<double>& ranges,
+    tiledb::sm::Layout layout,
+    tiledb_subarray_t** subarray,
+    bool coalesce_ranges);
+
+template void create_subarray<int8_t>(
+    const tiledb::Context* ctx,
+    const tiledb::Array* array,
+    const SubarrayRanges<int8_t>& ranges,
+    tiledb::sm::Layout layout,
+    tiledb::Subarray** subarray,
+    bool coalesce_ranges);
+template void create_subarray<uint8_t>(
+    const tiledb::Context* ctx,
+    const tiledb::Array* array,
+    const SubarrayRanges<uint8_t>& ranges,
+    tiledb::sm::Layout layout,
+    tiledb::Subarray** subarray,
+    bool coalesce_ranges);
+template void create_subarray<int16_t>(
+    const tiledb::Context* ctx,
+    const tiledb::Array* array,
+    const SubarrayRanges<int16_t>& ranges,
+    tiledb::sm::Layout layout,
+    tiledb::Subarray** subarray,
+    bool coalesce_ranges);
+template void create_subarray<uint16_t>(
+    const tiledb::Context* ctx,
+    const tiledb::Array* array,
+    const SubarrayRanges<uint16_t>& ranges,
+    tiledb::sm::Layout layout,
+    tiledb::Subarray** subarray,
+    bool coalesce_ranges);
+template void create_subarray<int32_t>(
+    const tiledb::Context* ctx,
+    const tiledb::Array* array,
+    const SubarrayRanges<int32_t>& ranges,
+    tiledb::sm::Layout layout,
+    tiledb::Subarray** subarray,
+    bool coalesce_ranges);
+template void create_subarray<uint32_t>(
+    const tiledb::Context* ctx,
+    const tiledb::Array* array,
+    const SubarrayRanges<uint32_t>& ranges,
+    tiledb::sm::Layout layout,
+    tiledb::Subarray** subarray,
+    bool coalesce_ranges);
+template void create_subarray<int64_t>(
+    const tiledb::Context* ctx,
+    const tiledb::Array* array,
+    const SubarrayRanges<int64_t>& ranges,
+    tiledb::sm::Layout layout,
+    tiledb::Subarray** subarray,
+    bool coalesce_ranges);
+template void create_subarray<uint64_t>(
+    const tiledb::Context* ctx,
+    const tiledb::Array* array,
+    const SubarrayRanges<uint64_t>& ranges,
+    tiledb::sm::Layout layout,
+    tiledb::Subarray** subarray,
+    bool coalesce_ranges);
+template void create_subarray<float>(
+    const tiledb::Context* ctx,
+    const tiledb::Array* array,
+    const SubarrayRanges<float>& ranges,
+    tiledb::sm::Layout layout,
+    tiledb::Subarray** subarray,
+    bool coalesce_ranges);
+template void create_subarray<double>(
+    const tiledb::Context* ctx,
+    const tiledb::Array* array,
+    const SubarrayRanges<double>& ranges,
+    tiledb::sm::Layout layout,
+    tiledb::Subarray** subarray,
+    bool coalesce_ranges);
+
 template void check_partitions<int8_t>(
     tiledb::sm::SubarrayPartitioner& partitioner,
     const std::vector<SubarrayRanges<int8_t>>& partitions,
