@@ -41,15 +41,20 @@
 #include <string>
 
 #include "tiledb/common/blank.h"
+#include "tiledb/common/common.h"
 #include "tiledb/common/logger_public.h"
 #include "tiledb/common/status.h"
+#include "tiledb/sm/enums/datatype.h"
+#include "tiledb/sm/misc/constants.h"
 #include "tiledb/sm/misc/types.h"
-#include "tiledb/sm/misc/utils.h"
-#include "tiledb/sm/query/query_buffer.h"
-#include "tiledb/sm/query/result_coords.h"
 #include "tiledb/sm/tile/tile.h"
 
 using namespace tiledb::common;
+using namespace tiledb::type;
+
+namespace tiledb::type {
+class Range;
+}
 
 namespace tiledb {
 namespace sm {
@@ -81,6 +86,24 @@ class Dimension {
    */
   Dimension(const std::string& name, Datatype type);
 
+  /**
+   * Constructor.
+   *
+   * @param name The name of the dimension.
+   * @param type The type of the dimension.
+   * @param cell_val_num The cell value number of the dimension.
+   * @param domain The range of the dimension range.
+   * @param filter_pipeline The filters of the dimension.
+   * @param tile_extent The tile extent of the dimension.
+   */
+  Dimension(
+      const std::string& name,
+      Datatype type,
+      uint32_t cell_val_num,
+      const Range& domain,
+      const FilterPipeline& filter_pipeline,
+      const ByteVecValue& tile_extent);
+
   /**
    * Constructor. It clones the input.
    *
@@ -112,16 +135,9 @@ class Dimension {
   Status set_cell_val_num(unsigned int cell_val_num);
 
   /** Returns the size (in bytes) of a coordinate in this dimension. */
-  uint64_t coord_size() const;
-
-  /**
-   *  Returns a coordinate in string format.
-   *
-   * @param buff The query buffer that contains all coordinates.
-   * @param i The position of the coordinate in the buffer.
-   * @return The coordinate in string format.
-   */
-  std::string coord_to_str(const QueryBuffer& buff, uint64_t i) const;
+  [[nodiscard]] inline size_t coord_size() const {
+    return datatype_size(type_);
+  }
 
   /**
    * Populates the object members from the data in the input binary buffer.
@@ -129,9 +145,10 @@ class Dimension {
    * @param buff The buffer to deserialize from.
    * @param type The type of the dimension.
    * @param version The array schema version.
-   * @return Status
+   * @return Status and Dimension
    */
-  Status deserialize(ConstBuffer* buff, uint32_t version, Datatype type);
+  static tuple<Status, optional<shared_ptr<Dimension>>> deserialize(
+      ConstBuffer* buff, uint32_t version, Datatype type);
 
   /** Returns the domain. */
   const Range& domain() const;
@@ -661,19 +678,17 @@ class Dimension {
       std::vector<uint64_t>& relevant_ranges);
 
   /** Compute covered on a set of relevant ranges. */
-  void covered_vec(
+  std::vector<bool> covered_vec(
       const NDRange& ranges,
       const Range& mbr,
-      const std::vector<uint64_t>& relevant_ranges,
-      std::vector<bool>& covered) const;
+      const std::vector<uint64_t>& relevant_ranges) const;
 
   /** Compute covered on a set of relevant ranges. */
   template <class T>
-  static void covered_vec(
+  static std::vector<bool> covered_vec(
       const NDRange& ranges,
       const Range& mbr,
-      const std::vector<uint64_t>& relevant_ranges,
-      std::vector<bool>& covered);
+      const std::vector<uint64_t>& relevant_ranges);
 
   /** Splits `r` at point `v`, producing 1D ranges `r1` and `r2`. */
   void split_range(
@@ -706,32 +721,6 @@ class Dimension {
   template <class T>
   static uint64_t tile_num(const Dimension* dim, const Range& range);
 
-  /**
-   * Maps the c-th cell in the input query buffer to a uint64 value,
-   * based on discretizing the domain from 0 to `max_bucket_val`.
-   * This value is used to compute a Hilbert value.
-   */
-  uint64_t map_to_uint64(
-      const QueryBuffer* buff,
-      uint64_t c,
-      uint64_t coords_num,
-      int bits,
-      uint64_t max_bucket_val) const;
-
-  /**
-   * Maps the c-th cell in the input query buffer to a uint64 value,
-   * based on discretizing the domain from 0 to `max_bucket_val`.
-   * This value is used to compute a Hilbert value.
-   */
-  template <class T>
-  static uint64_t map_to_uint64(
-      const Dimension* dim,
-      const QueryBuffer* buff,
-      uint64_t c,
-      uint64_t coords_num,
-      int bits,
-      uint64_t max_bucket_val);
-
   /**
    * Maps the input coordinate to a uint64 value,
    * based on discretizing the domain from 0 to `max_bucket_val`.
@@ -756,30 +745,6 @@ class Dimension {
       int bits,
       uint64_t max_bucket_val);
 
-  /**
-   * Maps the input result coordinate to a uint64 value,
-   * based on discretizing the domain from 0 to `max_bucket_val`.
-   * This value is used to compute a Hilbert value.
-   */
-  uint64_t map_to_uint64(
-      const ResultCoords& coord,
-      uint32_t dim_idx,
-      int bits,
-      uint64_t max_bucket_val) const;
-
-  /**
-   * Maps the input result coordinate to a uint64 value,
-   * based on discretizing the domain from 0 to `max_bucket_val`.
-   * This value is used to compute a Hilbert value.
-   */
-  template <class T>
-  static uint64_t map_to_uint64_3(
-      const Dimension* dim,
-      const ResultCoords& coord,
-      uint32_t dim_idx,
-      int bits,
-      uint64_t max_bucket_val);
-
   /**
    * Maps a uint64 value (produced by `map_to_uint64`) to its corresponding
    * value in the original dimension domain. `max_bucket_val` is the maximum
@@ -852,13 +817,19 @@ class Dimension {
   Status set_null_tile_extent_to_range();
 
   /** Returns the tile extent. */
-  const ByteVecValue& tile_extent() const;
+  inline const ByteVecValue& tile_extent() const {
+    return tile_extent_;
+  }
 
   /** Returns the dimension type. */
-  Datatype type() const;
+  inline Datatype type() const {
+    return type_;
+  }
 
   /** Returns true if the dimension is var-sized. */
-  bool var_size() const;
+  inline bool var_size() const {
+    return cell_val_num_ == constants::var_num;
+  }
 
  private:
   /* ********************************* */
@@ -988,11 +959,8 @@ class Dimension {
    * Stores the appropriate templated covered_vec() function based on the
    * dimension datatype.
    */
-  std::function<void(
-      const NDRange&,
-      const Range&,
-      const std::vector<uint64_t>&,
-      std::vector<bool>&)>
+  std::function<std::vector<bool>(
+      const NDRange&, const Range&, const std::vector<uint64_t>&)>
       covered_vec_func_;
 
   /**
@@ -1015,14 +983,6 @@ class Dimension {
    */
   std::function<uint64_t(const Dimension* dim, const Range&)> tile_num_func_;
 
-  /**
-   * Stores the appropriate templated map_to_uint64() function based on
-   * the dimension datatype.
-   */
-  std::function<uint64_t(
-      const Dimension*, const QueryBuffer*, uint64_t, uint64_t, int, uint64_t)>
-      map_to_uint64_func_;
-
   /**
    * Stores the appropriate templated map_to_uint64_2() function based on
    * the dimension datatype.
@@ -1031,14 +991,6 @@ class Dimension {
       const Dimension*, const void*, uint64_t, int, uint64_t)>
       map_to_uint64_2_func_;
 
-  /**
-   * Stores the appropriate templated map_to_uint64_3() function based on
-   * the dimension datatype.
-   */
-  std::function<uint64_t(
-      const Dimension*, const ResultCoords&, uint32_t, int, uint64_t)>
-      map_to_uint64_3_func_;
-
   /**
    * Stores the appropriate templated map_from_uint64() function based on
    * the dimension datatype.
@@ -1073,7 +1025,7 @@ class Dimension {
 
     // Upper bound should not be smaller than lower
     if (domain[1] < domain[0])
-      return LOG_STATUS(Status::DimensionError(
+      return LOG_STATUS(Status_DimensionError(
           "Domain check failed; Upper domain bound should "
           "not be smaller than the lower one"));
 
@@ -1081,7 +1033,7 @@ class Dimension {
     // for integer domains
     if (domain[0] == std::numeric_limits<T>::min() &&
         domain[1] == std::numeric_limits<T>::max())
-      return LOG_STATUS(Status::DimensionError(
+      return LOG_STATUS(Status_DimensionError(
           "Domain check failed; Domain range (upper + lower + 1) is larger "
           "than the maximum unsigned number"));
 
@@ -1102,14 +1054,14 @@ class Dimension {
     // Check for NAN and INF
     if (std::isinf(domain[0]) || std::isinf(domain[1]))
       return LOG_STATUS(
-          Status::DimensionError("Domain check failed; domain contains NaN"));
+          Status_DimensionError("Domain check failed; domain contains NaN"));
     if (std::isnan(domain[0]) || std::isnan(domain[1]))
       return LOG_STATUS(
-          Status::DimensionError("Domain check failed; domain contains NaN"));
+          Status_DimensionError("Domain check failed; domain contains NaN"));
 
     // Upper bound should not be smaller than lower
     if (domain[1] < domain[0])
-      return LOG_STATUS(Status::DimensionError(
+      return LOG_STATUS(Status_DimensionError(
           "Domain check failed; Upper domain bound should "
           "not be smaller than the lower one"));
 
@@ -1201,15 +1153,9 @@ class Dimension {
   /** Sets the templated tile_num() function. */
   void set_tile_num_func();
 
-  /** Sets the templated map_to_uint64() function. */
-  void set_map_to_uint64_func();
-
   /** Sets the templated map_to_uint64_2() function. */
   void set_map_to_uint64_2_func();
 
-  /** Sets the templated map_to_uint64_3() function. */
-  void set_map_to_uint64_3_func();
-
   /** Sets the templated map_from_uint64() function. */
   void set_map_from_uint64_func();
 
