@@ -61,7 +61,6 @@ class Deleter {
   /* ********************************* */
 
   Deleter() = default;
-  Deleter(const Deleter&) = default;
 
   /* ********************************* */
   /*              DELETERS             */
@@ -75,6 +74,10 @@ class Deleter {
     tiledb_array_free(&p);
   }
 
+  void operator()(tiledb_subarray_t* p) const {
+    tiledb_subarray_free(&p);
+  }
+
   void operator()(tiledb_query_t* p) const {
     tiledb_query_free(&p);
   }
@@ -123,6 +126,10 @@ class Deleter {
     tiledb_error_free(&p);
   }
 
+  void operator()(tiledb_group_t* p) const {
+    tiledb_group_free(&p);
+  }
+
  private:
   /* ********************************* */
   /*         PRIVATE ATTRIBUTES        */
