@@ -5,7 +5,7 @@
  *
  * The MIT License
  *
- * @copyright Copyright (c) 2017-2021 TileDB, Inc.
+ * @copyright Copyright (c) 2017-2022 TileDB, Inc.
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -71,11 +71,20 @@ const unsigned rtree_fanout = 10;
 /** The array schema file name. */
 const std::string array_schema_filename = "__array_schema.tdb";
 
-/** The array schema folder name. */
-const std::string array_schema_folder_name = "__schema";
+/** The array schema directory name. */
+const std::string array_schema_dir_name = "__schema";
 
-/** The array metadata folder name. */
-const std::string array_metadata_folder_name = "__meta";
+/** The array metadata directory name. */
+const std::string array_metadata_dir_name = "__meta";
+
+/** The array fragment metadata directory name. */
+const std::string array_fragment_meta_dir_name = "__fragment_meta";
+
+/** The array fragments directory name. */
+const std::string array_fragments_dir_name = "__fragments";
+
+/** The array commit directory name. */
+const std::string array_commits_dir_name = "__commits";
 
 /** The fragment metadata file name. */
 const std::string fragment_metadata_filename = "__fragment_metadata.tdb";
@@ -122,9 +131,6 @@ Compressor real_coords_compression = Compressor::ZSTD;
 /** The default compression level for the coordinates. */
 int coords_compression_level = -1;
 
-/** The filelock name. */
-const std::string filelock_name = "__lock.tdb";
-
 /** The special value for an empty int32. */
 const int empty_int32 = std::numeric_limits<int32_t>::min();
 
@@ -140,6 +146,9 @@ const double empty_float64 = std::numeric_limits<double>::quiet_NaN();
 /** The special value for an empty char. */
 const char empty_char = std::numeric_limits<char>::min();
 
+/** The special value for an empty blob. */
+constexpr std::byte empty_blob{0};
+
 /** The special value for an empty int8. */
 const int8_t empty_int8 = std::numeric_limits<int8_t>::min();
 
@@ -188,9 +197,18 @@ const std::string vacuum_file_suffix = ".vac";
 /** Suffix for the special ok files used in TileDB. */
 const std::string ok_file_suffix = ".ok";
 
+/** Suffix for the special write files used in TileDB. */
+const std::string write_file_suffix = ".wrt";
+
 /** Suffix for the special metadata files used in TileDB. */
 const std::string meta_file_suffix = ".meta";
 
+/** Suffix for the special commits files used in TileDB. */
+const std::string con_commits_file_suffix = ".con";
+
+/** Suffix for the special ignore files used in TileDB. */
+const std::string ignore_file_suffix = ".ign";
+
 /** Default datatype for a generic tile. */
 const Datatype generic_tile_datatype = Datatype::CHAR;
 
@@ -206,6 +224,12 @@ uint64_t generic_tile_cell_size = sizeof(char);
 /** The group file name. */
 const std::string group_filename = "__tiledb_group.tdb";
 
+/** The group details directory name. */
+const std::string group_detail_dir_name = "__group";
+
+/** The group metadata directory name. */
+const std::string group_metadata_dir_name = "__meta";
+
 /** The maximum number of bytes written in a single I/O. */
 const uint64_t max_write_bytes = std::numeric_limits<int>::max();
 
@@ -325,6 +349,9 @@ const std::string filter_checksum_md5_str = "CHECKSUM_MD5";
 /** String describing FILTER_CHECKSUM_SHA256. */
 const std::string filter_checksum_sha256_str = "CHECKSUM_SHA256";
 
+/** String describing FILTER_DICTIONARY. */
+const std::string filter_dictionary_str = "DICTIONARY_ENCODING";
+
 /** The string representation for FilterOption type compression_level. */
 const std::string filter_option_compression_level_str = "COMPRESSION_LEVEL";
 
@@ -352,6 +379,9 @@ const std::string float64_str = "FLOAT64";
 /** The string representation for type char. */
 const std::string char_str = "CHAR";
 
+/** The string representation for type blob. */
+const std::string blob_str = "BLOB";
+
 /** The string representation for type int8. */
 const std::string int8_str = "INT8";
 
@@ -525,7 +555,7 @@ const int32_t library_version[3] = {
     TILEDB_VERSION_MAJOR, TILEDB_VERSION_MINOR, TILEDB_VERSION_PATCH};
 
 /** The TileDB serialization format version number. */
-const uint32_t format_version = 10;
+const uint32_t format_version = 13;
 
 /** The lowest version supported for back compat writes. */
 const uint32_t back_compat_writes_min_format_version = 7;
@@ -572,8 +602,44 @@ const unsigned concurrent_attr_reads = 2;
 /** The redirection header key in REST response. */
 extern const std::string redirection_header_key = "location";
 
+/** String describing MIME_AUTODETECT. */
+const std::string mime_autodetect_str = "AUTODETECT";
+
+/** String describing MIME_TIFF. */
+const std::string mime_tiff_str = "image/tiff";
+
+/** String describing MIME_PDF. */
+const std::string mime_pdf_str = "application/pdf";
+
+/** The default tile extent used in filestore arrays. */
+const uint64_t filestore_default_tile_extent = 1024;
+
+/** Name of the single dimension used in filestore arrays. */
+const std::string filestore_dimension_name = "position";
+
+/** Name of the single attribute used in filestore arrays. */
+const std::string filestore_attribute_name = "contents";
+
+/** Name of the metadata key used in filestore arrays for current size. */
+const std::string filestore_metadata_size_key = "file_size";
+
+/** Name of the metadata key used in filestore arrays for mime type. */
+const std::string filestore_metadata_mime_type_key = "mime_type";
+
+/** Name of the metadata key used in filestore arrays for mime encoding. */
+const std::string filestore_metadata_mime_encoding_key = "mime_encoding";
+
+/** Name of the metadata key used in filestore arrays for original filename. */
+const std::string filestore_metadata_original_filename_key =
+    "original_file_name";
+
+/** Name of the metadata key used in filestore arrays for filename extension. */
+const std::string filestore_metadata_file_extension_key = "file_extension";
+
 const void* fill_value(Datatype type) {
   switch (type) {
+    case Datatype::BLOB:
+      return &constants::empty_blob;
     case Datatype::INT8:
       return &constants::empty_int8;
     case Datatype::UINT8:
