@@ -35,6 +35,7 @@
 #include "tiledb/sm/array_schema/dimension.h"
 #include "tiledb/sm/buffer/buffer.h"
 #include "tiledb/sm/enums/datatype.h"
+#include "tiledb/sm/misc/tdb_math.h"
 #include "tiledb/sm/misc/utils.h"
 
 #include <cassert>
@@ -127,15 +128,12 @@ unsigned RTree::dim_num() const {
   return (domain_ == nullptr) ? 0 : domain_->dim_num();
 }
 
-const Domain* RTree::domain() const {
-  return domain_;
-}
-
 unsigned RTree::fanout() const {
   return fanout_;
 }
 
-TileOverlap RTree::get_tile_overlap(const NDRange& range) const {
+TileOverlap RTree::get_tile_overlap(
+    const NDRange& range, std::vector<bool>& is_default) const {
   TileOverlap overlap;
 
   // Empty tree
@@ -155,7 +153,7 @@ TileOverlap RTree::get_tile_overlap(const NDRange& range) const {
     const auto& mbr = levels_[entry.level_][entry.mbr_idx_];
 
     // Get overlap ratio
-    auto ratio = domain_->overlap_ratio(range, mbr);
+    auto ratio = domain_->overlap_ratio(range, is_default, mbr);
 
     // If there is overlap
     if (ratio != 0.0) {
@@ -292,12 +290,11 @@ Status RTree::serialize(Buffer* buff) const {
 
 Status RTree::set_leaf(uint64_t leaf_id, const NDRange& mbr) {
   if (levels_.size() != 1)
-    return LOG_STATUS(Status::RTreeError(
+    return LOG_STATUS(Status_RTreeError(
         "Cannot set leaf; There are more than one levels in the tree"));
 
   if (leaf_id >= levels_[0].size())
-    return LOG_STATUS(
-        Status::RTreeError("Cannot set leaf; Invalid lead index"));
+    return LOG_STATUS(Status_RTreeError("Cannot set leaf; Invalid lead index"));
 
   levels_[0][leaf_id] = mbr;
 
@@ -318,8 +315,8 @@ Status RTree::set_leaf_num(uint64_t num) {
 
   if (num < levels_[0].size())
     return LOG_STATUS(
-        Status::RTreeError("Cannot set number of leaves; provided number "
-                           "cannot be smaller than the current leaf number"));
+        Status_RTreeError("Cannot set number of leaves; provided number "
+                          "cannot be smaller than the current leaf number"));
 
   levels_[0].resize(num);
   return Status::Ok();
@@ -397,6 +394,7 @@ Status RTree::deserialize_v1_v4(ConstBuffer* cbuff, const Domain* domain) {
 Status RTree::deserialize_v5(ConstBuffer* cbuff, const Domain* domain) {
   RETURN_NOT_OK(cbuff->read(&fanout_, sizeof(fanout_)));
   unsigned level_num;
+
   RETURN_NOT_OK(cbuff->read(&level_num, sizeof(level_num)));
 
   if (level_num == 0)
@@ -405,16 +403,18 @@ Status RTree::deserialize_v5(ConstBuffer* cbuff, const Domain* domain) {
   levels_.clear();
   levels_.resize(level_num);
   auto dim_num = domain->dim_num();
+
   uint64_t mbr_num;
   for (unsigned l = 0; l < level_num; ++l) {
     RETURN_NOT_OK(cbuff->read(&mbr_num, sizeof(uint64_t)));
     levels_[l].resize(mbr_num);
+
     for (uint64_t m = 0; m < mbr_num; ++m) {
       levels_[l][m].resize(dim_num);
       for (unsigned d = 0; d < dim_num; ++d) {
-        auto dim = domain_->dimension(d);
+        auto dim = domain->dimension(d);
         if (!dim->var_size()) {  // Fixed-sized
-          auto r_size = 2 * domain->dimension(d)->coord_size();
+          auto r_size = 2 * dim->coord_size();
           levels_[l][m][d].set_range(cbuff->cur_data(), r_size);
           cbuff->advance_offset(r_size);
         } else {  // Var-sized
