@@ -38,11 +38,13 @@
 #include "tiledb/sm/array_schema/domain.h"
 #include "tiledb/sm/enums/layout.h"
 #include "tiledb/sm/misc/types.h"
+#include "tiledb/type/range/range.h"
 
 #include <cassert>
 #include <iostream>
 
 using namespace tiledb::common;
+using namespace tiledb::type;
 
 namespace tiledb {
 namespace sm {
@@ -62,9 +64,9 @@ CellSlabIter<T>::CellSlabIter(const Subarray* subarray)
     : subarray_(subarray) {
   end_ = true;
   if (subarray != nullptr) {
-    auto array_schema = subarray->array()->array_schema_latest();
-    auto dim_num = array_schema->dim_num();
-    auto coord_size = array_schema->dimension(0)->coord_size();
+    const auto& array_schema = subarray->array()->array_schema_latest();
+    auto dim_num = array_schema.dim_num();
+    auto coord_size = array_schema.dimension(0)->coord_size();
     aux_tile_coords_.resize(dim_num);
     aux_tile_coords_2_.resize(dim_num * coord_size);
   }
@@ -245,18 +247,18 @@ template <class T>
 Status CellSlabIter<T>::init_ranges() {
   // For easy reference
   auto dim_num = subarray_->dim_num();
-  auto array_schema = subarray_->array()->array_schema_latest();
-  auto array_domain = array_schema->domain()->domain();
+  const auto& array_schema = subarray_->array()->array_schema_latest();
+  auto array_domain = array_schema.domain().domain();
   uint64_t range_num;
   T tile_extent, dim_domain_start;
-  const tiledb::sm::Range* r;
+  const tiledb::type::Range* r;
 
   ranges_.resize(dim_num);
   for (unsigned d = 0; d < dim_num; ++d) {
     auto dim_dom = (const T*)array_domain[d].data();
     RETURN_NOT_OK(subarray_->get_range_num(d, &range_num));
     ranges_[d].reserve(range_num);
-    tile_extent = *(const T*)array_schema->domain()->tile_extent(d).data();
+    tile_extent = *(const T*)array_schema.domain().tile_extent(d).data();
     dim_domain_start = dim_dom[0];
     for (uint64_t j = 0; j < range_num; ++j) {
       RETURN_NOT_OK(subarray_->get_range(d, j, &r));
@@ -275,14 +277,14 @@ Status CellSlabIter<T>::sanity_check() const {
   // Check layout
   auto layout = subarray_->layout();
   if (layout != Layout::ROW_MAJOR && layout != Layout::COL_MAJOR)
-    return LOG_STATUS(Status::CellSlabIterError(
+    return LOG_STATUS(Status_CellSlabIterError(
         "Unsupported subarray layout; the iterator supports only row-major and "
         "column-major layouts"));
 
   // Check type
   bool error;
-  auto array_schema = subarray_->array()->array_schema_latest();
-  auto type = array_schema->domain()->dimension(0)->type();
+  const auto& array_schema = subarray_->array()->array_schema_latest();
+  auto type = array_schema.domain().dimension(0)->type();
   switch (type) {
     case Datatype::INT8:
       error = !std::is_same<T, int8_t>::value;
@@ -338,7 +340,7 @@ Status CellSlabIter<T>::sanity_check() const {
   }
 
   if (error)
-    return LOG_STATUS(Status::CellSlabIterError(
+    return LOG_STATUS(Status_CellSlabIterError(
         "Datatype mismatch between cell slab iterator and subarray"));
 
   return Status::Ok();
